@@ -850,8 +850,10 @@ gog_object_set_id (GogObject *obj, unsigned id)
 
 	g_return_if_fail (IS_GOG_OBJECT (obj));
 
-	if (id == 0)
-		return gog_object_generate_id (obj);
+	if (id == 0) {
+		gog_object_generate_id (obj);
+		return;
+	}
 
 	g_return_if_fail (GOG_OBJECT (obj)->parent != NULL);
 
@@ -900,7 +902,7 @@ dataset_dup (GogDataset const *src, GogDataset *dst)
 GogObject *
 gog_object_dup (GogObject const *src, GogObject *new_parent, GogDataDuplicator datadup)
 {
-	gint	     n;
+	guint	     n;
 	GParamSpec **props;
 	GogObject   *dst = NULL;
 	GSList      *ptr;
@@ -1547,12 +1549,19 @@ gog_object_add_by_role (GogObject *parent, GogObjectRole const *role, GogObject
 
 	g_return_val_if_fail (is_a != 0, NULL);
 
-	if (child == NULL)
+	if (child == NULL) {
 		child = (role->allocate)
 			? (role->allocate) (parent)
 			: g_object_new (is_a, NULL);
 
+		/* g_object_new or the allocator have already generated an
+		 * error message, just exit */
+		if (child == NULL)
+			return NULL;
+	}
+
 	g_return_val_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (child, is_a), NULL);
+
 	child->explicitly_typed_role = explicitly_typed_role;
 	if (gog_object_set_parent (child, parent, role, 0))
 		return child;
