@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -28,7 +28,6 @@
 #include <goffice/graph/gog-data-set.h>
 #include <goffice/graph/gog-style.h>
 #include <glib-object.h>
-#include <gtk/gtknotebook.h>
 
 G_BEGIN_DECLS
 
@@ -56,7 +55,7 @@ typedef enum {
 	GOG_SERIES_ERRORS
 } GogSeriesPriority;
 
-#define GOG_SERIES_ACCEPT_REGRESSION_CURVE	1
+#define GOG_SERIES_ACCEPT_TREND_LINE	1
 
 struct _GogSeriesDimDesc {
 	char const *name;
@@ -95,6 +94,8 @@ typedef struct {
 
 	/* Virtuals */
 	void (*dim_changed) (GogSeries *series, int dim_i);
+	unsigned (*get_xy_data) (GogSeries const *series,
+					double const **x, double const **y);
 } GogSeriesClass;
 
 #define GOG_SERIES_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST ((k), GOG_SERIES_TYPE, GogSeriesClass))
